@@ -5,8 +5,8 @@
 
       parameter (maxcli = 256)
 
-      character(len=*)  ccfile
-      character(len=64) cmsg
+      character ccfile*(*)
+      character cmsg*64
 
       INTEGER ir(maxcli),ig(maxcli),ib(maxcli)
 
