@@ -333,25 +333,13 @@
       ENTRY SWSCLI(ICLIDX,LFCATR)
 
 *     / SET LINE COLOR INDEX /
-      IF(.NOT.LFCATR) THEN
+
 
       IF (IWS.EQ.1) CALL ZMSCLI(ICLIDX)
       IF (IWS.EQ.2) CALL ZMSCLI(ICLIDX)
       IF (IWS.EQ.3) CALL Z1SCLI(ICLIDX)
       IF (IWS.EQ.4) CALL ZGSCLI(ICLIDX)
 
-      ELSE
-
-      CALL SGQCL(ICLIDX,NR,NB,NG)
-      CALL UIFPAC(NR,NG,NB,NCOLOR)
-
-      IF (IWS.EQ.1) CALL ZMSLCL(NCOLOR)
-      IF (IWS.EQ.2) CALL ZMSLCL(NCOLOR)
-      IF (IWS.EQ.3) CALL Z1SLCL(NCOLOR)
-      IF (IWS.EQ.4) CALL ZGSLCL(NCOLOR)
-
-      ENDIF
-
       RETURN
 *-----------------------------------------------------------------------
       ENTRY SWSLCL(ICOLOR)
