@@ -277,13 +277,11 @@
       ENTRY SWSLCL(ICOLOR)
 
 *     / SET LINE COLOR IN 24BIT RGB /
-      CALL SGQCL(ICLIDX,NR,NB,NG)
-      CALL UIFPAC(NR,NG,NB,NCOLOR)
 
-      IF (IWS.EQ.1) CALL ZXSLCL(NCOLOR)
-      IF (IWS.EQ.2) CALL Z1SLCL(NCOLOR)
+      IF (IWS.EQ.1) CALL ZXSLCL(ICOLOR)
+      IF (IWS.EQ.2) CALL Z1SLCL(ICOLOR)
 *      IF (IWS.EQ.3) CALL ZTSLCL(NCOLOR)
-      IF (IWS.EQ.4) CALL ZGSLCL(NCOLOR)
+      IF (IWS.EQ.4) CALL ZGSLCL(ICOLOR)
 
       RETURN
 *-----------------------------------------------------------------------
