@@ -7,6 +7,8 @@
 
 #undef DUMP_IMAGE
 
+//#define DEBUG_MODE
+
 #ifdef HAVE_GTK_H
   #include <gtk/gtk.h>
   #include <gdk/gdkkeysyms.h>
@@ -59,6 +61,7 @@ static int iwtroz;           /* direction of frame */
 static int lclatrz;          /* type of screen */
 static int wsxwd, wsywd, wsxmnz, wsxmxz, wsymnz, wsymxz;
 static int ixz, iyz, iwz, ihz, ixxz, iyyz, page; /* for image */
+static int irxt,iryt,irxb,iryb,irwidth; /* for Invalidate */
 static int posx, posy, wait_np, wait_op, wait_cl, key, dump, fgbg;
 static int nbmap, nn1[MAXBMP], nn2[MAXBMP], nx[MAXBMP], ny[MAXBMP];
 static char dmpfile[80], xtitle[80];
@@ -90,21 +93,44 @@ static GdkColor cx[MAXCLI];
 static int ifunc, next;
 static int dumpz = FALSE;
 
+void zguprect(int,int);
+
 /*------------------------- GDK 設定 ------------------------*/
 
-void zgupdate(void)
+void zgupdate(int flag)
 {
-    GdkRectangle update_rect;
 
+    GdkRectangle update_rect;
+// GdkRegion *region;
+
+    update_rect.x = irxb-1;
+    update_rect.y = iryb-1;
+    update_rect.width = irxt-irxb+1;
+    update_rect.height = iryt-iryb+1;
+    
+    if( irxb == 0 && irxt == 0 && iryb == 0 && iryt == 0 && flag == 0){
     update_rect.x = 0;
     update_rect.y = 0;
     update_rect.width = drawing_area->allocation.width;
     update_rect.height = drawing_area->allocation.height;
+	}
 
+#ifdef DEBUG_MODE
+	printf("zgupdate %d\n");
+#endif
     while (gtk_events_pending())
         gtk_main_iteration();
-
     gtk_widget_draw(drawing_area, &update_rect);
+   //     if (GTK_WIDGET_VISIBLE(window) && GTK_WIDGET_DRAWABLE(drawing_area))
+   // gdk_window_process_updates(drawing_area->window, FALSE);
+
+//    if (flag == 1){
+//    gtk_widget_draw(drawing_area, &update_rect);
+//    }
+     irxt = 0;
+     irxb = 0;
+     iryt = 0;
+     iryb = 0;
 
 }
 
@@ -112,6 +138,9 @@ void zgsdrw_(GtkWidget *d)
 {
 
   /* set/reset drawing_area */
+#ifdef DEBUG_MODE
+	printf("zgsdrw\n");
+#endif
 
   drawing_area = d;
 }
@@ -126,12 +155,19 @@ gint zg_expose_event(GtkWidget *widget, GdkEventExpose *event)
 		  event->area.x, event->area.y,
 		  event->area.x, event->area.y,
 		  event->area.width, event->area.height);
+#ifdef DEBUG_MODE
+	printf("zg_expose_event\n");
+#endif
 
   return FALSE;
 }
 
 gint zg_key_press_event(GtkWidget *widget, GdkEventKey *event)
 {
+#ifdef DEBUG_MODE
+	printf("zg_key_press_event\n");
+#endif
+
   switch (ifunc){
   case DOPN:
     next = TRUE;
@@ -179,6 +215,10 @@ gint zg_key_press_event(GtkWidget *widget, GdkEventKey *event)
 
 gint zg_button_press_event(GtkWidget *widget, GdkEventButton *event)
 {
+#ifdef DEBUG_MODE
+	printf("zg_button_press_event\n");
+#endif
+
   switch (ifunc){
   case DOPN:
   case DCLS:
@@ -198,6 +238,9 @@ static GtkWidget *zggdrw_(DCL_INT *width, DCL_INT *height)
 {
   GtkWidget * drw;
   GtkWidget *vbox;
+#ifdef DEBUG_MODE
+	printf("zggdrw\n");
+#endif
 
   wsxwd  = *width  + 2 * PAD;  /* window width */
   wsywd  = *height + 2 * PAD;  /* window height */
@@ -242,6 +285,9 @@ static GtkWidget *zggdrw_(DCL_INT *width, DCL_INT *height)
 
 GtkWidget *zgqdrw_(void)
 {
+#ifdef DEBUG_MODE
+	printf("zgqdrw\n");
+#endif
   return drawing_area;
 }
 
@@ -249,6 +295,9 @@ void zgspmp_(GdkPixmap *pmap)
 {
 
   /* set/reset pixmap */
+#ifdef DEBUG_MODE
+	printf("zgspmp\n");
+#endif
 
   pixmap = pmap;
 }
@@ -256,6 +305,9 @@ void zgspmp_(GdkPixmap *pmap)
 static GdkPixmap *zggpmp(void)
 {
   GdkPixmap *pmap;
+#ifdef DEBUG_MODE
+	printf("zggpmp\n");
+#endif
 
   pmap = gdk_pixmap_new(drawing_area->window,
 			drawing_area->allocation.width,
@@ -274,6 +326,9 @@ static GdkPixmap *zggpmp(void)
 
 GdkPixmap *zgqpmp_(void)
 {
+#ifdef DEBUG_MODE
+	printf("zgqpmp\n");
+#endif
   return pixmap;
 }
 
@@ -283,6 +338,9 @@ static void zgbmcv(int *nx, int *ny, char bmline[], char bitmap[])
   static int n8 = 8;
   int n, nb;
   unsigned bx;
+#ifdef DEBUG_MODE
+	printf("zgbmcv\n");
+#endif
 
   nb = *nx * *ny / n8;
   for (n = 0; n < nb; n++){
@@ -316,6 +374,10 @@ void zgdopn_(DCL_INT *width, DCL_INT *height, DCL_INT *iposx, DCL_INT *iposy,
   GdkRectangle update_rect;
   GdkEvent *ev;
 
+#ifdef DEBUG_MODE
+	printf("zgdopn\n");
+#endif
+
   posx      = *iposx;
   posy      = *iposy;
   wait_np   = *lwait;
@@ -447,7 +509,7 @@ void zgdopn_(DCL_INT *width, DCL_INT *height, DCL_INT *iposx, DCL_INT *iposy,
 
     pixmap = zggpmp();
 
-    zgupdate();
+    zgupdate(0);
 
     if (wait_op) {
       next = FALSE;
@@ -472,10 +534,13 @@ void zgdcls_(void)
 
   GdkEvent *ev;
 
+#ifdef DEBUG_MODE
+	printf("zgdcls\n");
+#endif
+
   if (ldclonly) {
-
-    zgupdate();
-
+    zgupdate(0);
+  }
     if (!wait_np && wait_cl) {
       next = FALSE;
       ifunc = DCLS;
@@ -485,13 +550,10 @@ void zgdcls_(void)
 	  break;
       }
     }
-
     gtk_widget_destroy(drawing_area);
     drawing_area = NULL;
     gtk_widget_destroy(window);
     window = NULL;
-
-  }
 #endif
 }
 
@@ -504,6 +566,10 @@ void zgpopn_(void)
   /* background drawing proc. */
   ++page;
 
+#ifdef DEBUG_MODE
+	printf("zgpopn\n");
+#endif
+
   iwdidz = 1;
   iclidz = 1;
 
@@ -530,9 +596,13 @@ void zgpcls_(void)
 
   char cnum[4];
 
+#ifdef DEBUG_MODE
+	printf("zgpcls\n");
+#endif
+
   if (ldclonly) {
 
-    zgupdate();
+    zgupdate(0);
 
     if (wait_np) {
       dumpz = FALSE;
@@ -603,6 +673,10 @@ void zgoopn_(char *objname, char *comment)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgoopn\n");
+#endif
+
   /* for long-type message? */
 #endif
 }
@@ -610,8 +684,13 @@ void zgoopn_(char *objname, char *comment)
 void zgocls_(char *objname)
 {
 #ifdef HAVE_GTK_H
-  if(!laltz) zgupdate();
+#ifdef DEBUG_MODE
+	printf("zgocls\n");
+#endif
 
+  if(!laltz){
+    zgupdate(1);
+ }
   /* for long-type message? */
 #endif
 }
@@ -625,6 +704,11 @@ void zgswdi_(DCL_INT *iwdidx)
   /* set line width index */
 
   iwdidz = *iwdidx % 10;
+  irwidth = iwdidz;
+
+#ifdef DEBUG_MODE
+	printf("zgswdi\n");
+#endif
 
   if (iwdidz == 0)
     iwdidz = 1;
@@ -635,6 +719,10 @@ void zgscli_(DCL_INT *iclidx)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgscli\n");
+#endif
+
   /* set line color index */
 
   iclidz = *iclidx % MAXCLI;
@@ -650,6 +738,10 @@ void zggopn_(void)
 
   /* open graphic segment */
 
+#ifdef DEBUG_MODE
+	printf("zggopn\n");
+#endif
+
   if (lfcmod == FALSE){
     gdk_gc_set_foreground(gc,&cx[iclidz]);
   }
@@ -665,11 +757,19 @@ void zggopn_(void)
 void zggmov_(DCL_REAL *wx, DCL_REAL *wy)
 {
 #ifdef HAVE_GTK_H
+  void zgfint_();
+  DCL_INT iwxold, iwyold;
 
   /* pen-up move */
 
+#ifdef DEBUG_MODE
+	printf("zggmov\n");
+#endif
+
   rwxold = *wx;
   rwyold = *wy;
+  zgfint_(&rwxold, &rwyold, &iwxold, &iwyold);
+  zguprect(iwxold,iwyold);
 #endif
 }
 
@@ -681,6 +781,10 @@ void zggplt_(DCL_REAL *wx, DCL_REAL *wy)
 
   /* pen-down move */
 
+#ifdef DEBUG_MODE
+	printf("zggplt\n");
+#endif
+
   zgfint_(&rwxold, &rwyold, &iwxold, &iwyold);
   zgfint_( wx,      wy,     &iwx,    &iwy   );
 
@@ -689,6 +793,7 @@ void zggplt_(DCL_REAL *wx, DCL_REAL *wy)
   }else{
     gdk_draw_line(pixmap, gc, (int) iwxold, (int) iwyold, (int) iwx, (int) iwy);
   }
+  zguprect(iwx,iwy);
   rwxold = *wx;
   rwyold = *wy;
 #endif
@@ -698,6 +803,10 @@ void zggcls_(void)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zggcls\n");
+#endif
+
   /* close graphic segment */
 #endif
 }
@@ -719,6 +828,10 @@ void zggton_(DCL_INT *np, DCL_REAL wpx[], DCL_REAL wpy[], DCL_INT *itpat)
 
   /* hard fill */
 
+#ifdef DEBUG_MODE
+	printf("zggton\n");
+#endif
+
   if (ltfrst) {
     gct = gdk_gc_new (drawing_area->window);
 
@@ -765,6 +878,7 @@ void zggton_(DCL_INT *np, DCL_REAL wpx[], DCL_REAL wpy[], DCL_INT *itpat)
     zgfint_(&wpx[i],&wpy[i], &ipx, &ipy);
     p[i].x = ipx;
     p[i].y = ipy;
+    zguprect(ipx,ipy);
   }
 
   gdk_draw_polygon(pixmap, gct, TRUE, p, *np);
@@ -776,6 +890,11 @@ void zggton_(DCL_INT *np, DCL_REAL wpx[], DCL_REAL wpy[], DCL_INT *itpat)
 void zgiopn_(DCL_INT *iwx, DCL_INT *iwy, DCL_INT *iwidth, DCL_INT *iheight)
 {
 #ifdef HAVE_GTK_H
+
+#ifdef DEBUG_MODE
+	printf("zgiopn\n");
+#endif
+
   ixz = *iwx;
   iyz = *iwy;
   iwz = *iwidth;
@@ -791,10 +910,15 @@ void zgidat_(DCL_INT image[], DCL_INT *nlen)
 #ifdef HAVE_GTK_H
   int i;
 
+#ifdef DEBUG_MODE
+	printf("zgidat\n");
+#endif
+
   for (i = 0; i < *nlen; i++) {
     if (image[i] != 0) {
       gdk_gc_set_foreground(gc, &cx[image[i]]);
       gdk_draw_point(pixmap, gc, ixxz, iyyz);
+      zguprect(ixxz,iyyz);
     }
     ixxz = ixxz + 1;
     if (ixxz >= ixz+iwz) {
@@ -806,6 +930,7 @@ void zgidat_(DCL_INT image[], DCL_INT *nlen)
       }
     }
   }
+  zgupdate(1);
 #endif
 }
 
@@ -813,6 +938,10 @@ void zgicls_(void)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgicls\n");
+#endif
+
   printf (" end\n");
 #endif
 }
@@ -823,6 +952,10 @@ void zgqpnt_(DCL_REAL *wx, DCL_REAL *wy, DCL_INT *mb)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgqpnt\n");
+#endif
+
   /* mouse pointer position info. */
 #endif
 }
@@ -835,6 +968,10 @@ void zgfint_(DCL_REAL *wx, DCL_REAL *wy, DCL_INT *iwx, DCL_INT *iwy)
   int dheight;
   int dwidth;
 
+#ifdef DEBUG_MODE
+	printf("zgfint\n");
+#endif
+
   *iwx = *wx + 0.5;
   *iwy = drawing_area->allocation.height - *wy + 0.5;
 #endif
@@ -843,6 +980,9 @@ void zgfint_(DCL_REAL *wx, DCL_REAL *wy, DCL_INT *iwx, DCL_INT *iwy)
 void zgiint_(DCL_INT *iwx, DCL_INT *iwy, DCL_REAL *wx, DCL_REAL *wy)
 {
 #ifdef HAVE_GTK_H
+#ifdef DEBUG_MODE
+	printf("zgiint\n");
+#endif
   *wx = *iwx;
   *wy = drawing_area->allocation.height - *iwy;
 #endif
@@ -854,6 +994,10 @@ void zgqwdc_(DCL_INT *lwdatr)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgwqdc\n");
+#endif
+
   /* inquire line width capability */
 
   *lwdatr = LWDATR;
@@ -864,6 +1008,10 @@ void zgqclc_(DCL_INT *lclatr)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgqclc\n");
+#endif
+
   /* inquire line color capability */
 
   *lclatr = lclatrz;
@@ -874,6 +1022,10 @@ void zgqtnc_(DCL_INT *ltnatr)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgqtnc\n");
+#endif
+
   /* inquire hard fill capability */
 
   *ltnatr = LTNATR;
@@ -884,6 +1036,10 @@ void zgqimc_(DCL_INT *limatr)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgqimc\n");
+#endif
+
   /* inquire bit image capability */
 
   *limatr = LIMATR;
@@ -894,6 +1050,10 @@ void zgqptc_(DCL_INT *lptatr)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgqptc\n");
+#endif
+
   /* inquire mouse point capability */
 
   *lptatr = LPTATR;
@@ -904,6 +1064,10 @@ void zgqrct_(DCL_REAL *wsxmn, DCL_REAL *wsxmx, DCL_REAL *wsymn, DCL_REAL *wsymx,
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgqrct\n");
+#endif
+
   /* inquire workstation rectangle */
 
   *wsxmn = 0.;
@@ -918,6 +1082,10 @@ void zgsrot_(DCL_INT *iwtrot)
 {
 #ifdef HAVE_GTK_H
 
+#ifdef DEBUG_MODE
+	printf("zgsrot\n");
+#endif
+
   /* set frame rotation flag */
 
   iwtroz = *iwtrot;
@@ -928,6 +1096,10 @@ void zgsrot_(DCL_INT *iwtrot)
 // New function for Fullcolor
 void zgsfcm_(DCL_INT *lfc)
 {
+#ifdef DEBUG_MODE
+	printf("zgsfcm\n");
+#endif
+
   lfcmod = *lfc;
 }
 void zgslcl_(DCL_INT *icolor)
@@ -935,6 +1107,10 @@ void zgslcl_(DCL_INT *icolor)
   GdkColor loccx;
   GdkColormap *cmap;
 
+#ifdef DEBUG_MODE
+	printf("zgslcl\n");
+#endif
+
   loccx.red   = ((*icolor >> 16) & 255) * 256;
   loccx.green = ((*icolor >> 8)  & 255) * 256;
   loccx.blue  = ((*icolor >> 0)  & 255) * 256;
@@ -953,6 +1129,10 @@ void zgstcl_(DCL_INT *icolor)
   GdkColor  loccx;
   GdkColormap *cmap;
 
+#ifdef DEBUG_MODE
+	printf("zgstcl\n");
+#endif
+
   if (ltfrst) {
     gct = gdk_gc_new (drawing_area->window);
 
@@ -974,6 +1154,10 @@ void zgiclr_(DCL_INT *image,DCL_INT *nlen)
   GdkColormap *cmap;
   int i;
 
+#ifdef DEBUG_MODE
+	printf("zgiclr\n");
+#endif
+
   for (i = 0; i < *nlen; i++) {
 
     loccx.red   = ((image[i] >> 16) & 255) * 256;
@@ -985,6 +1169,7 @@ void zgiclr_(DCL_INT *image,DCL_INT *nlen)
     gdk_gc_set_foreground(gc,&loccx);
 
     gdk_draw_point(pixmap,gc,(int) ixxz,(int) iyyz);
+    zguprect(ixxz,iyyz);
 
     ixxz = ixxz + 1;
 
@@ -997,6 +1182,7 @@ void zgiclr_(DCL_INT *image,DCL_INT *nlen)
       }
     }
   }
+  zgupdate(1);
 }
 
 
@@ -1016,6 +1202,10 @@ void zgclini_(char clrmap[], DCL_INT *lfgbg)
 
   /* read colormap file */
 
+#ifdef DEBUG_MODE
+	printf("zgclini\n");
+#endif
+
   cfnchr(cmapz, clrmap, 79);
 
   if ((stream = fopen(cmapz, "r")) == NULL) {
@@ -1067,4 +1257,9 @@ void zgclini_(char clrmap[], DCL_INT *lfgbg)
 
 }
 
-
+void zguprect(int ix,int iy){
+	if ((ix-irwidth) < irxb || irxb == 0) irxb =ix-irwidth;
+	if ((ix+irwidth) > irxt ) irxt =ix+irwidth;
+	if ((iy-irwidth) < iryb || iryb == 0) iryb =iy-irwidth;
+	if ((iy+irwidth) > iryt ) iryt =iy+irwidth;
+}
