@@ -1,11 +1,11 @@
 *-----------------------------------------------------------------------
       SUBROUTINE UIPDA3(U, V, W, MX, NX, NY)
 
-      PARAMETER (MAXPXL=1000)
-      REAL      U(MX, *), V(MX, *)
+      PARAMETER (MAXPXL=4000)
+      REAL      U(MX, *), V(MX, *), W(MX, *) 
       INTEGER   IMAGE(MAXPXL)
 
-      CALL UIPD2Z(U, V, W, MX, NX, NY, IMAGE, MAXPXL)
+      CALL UIPD3Z(U, V, W, MX, NX, NY, IMAGE, MAXPXL)
 
       END
 *-----------------------------------------------------------------------
